@@ -40,6 +40,7 @@ from . import html_tools
 from . import timeout
 from . import logging
 from . import time_utils
+from . import phrasing
 
 
 #---------#
@@ -49,8 +50,8 @@ from . import time_utils
 AUGMENTATION_ORDER = [
     "with_module_decorations",
     "tracing_function_calls",
-    "with_setup",
     "with_cleanup",
+    "with_setup", # must be below with_cleanup!
     "capturing_printed_output",
     "with_fake_input",
     "with_timeout",
@@ -157,12 +158,14 @@ def create_module_import_payload(
             sandbox=None
         )
 
-        result = {
+        # Create result as a copy of the base context
+        result = copy.copy(context)
+        result.update({
             "original_source": original_source,
             "source": fixed,
             "scope": node,
             "module": module
-        }
+        })
         if errors:
             result["parse_errors"] = errors
 
@@ -202,10 +205,14 @@ def create_read_variable_payload(varname):
                 )
                 logging.log(traceback.format_exc())
                 raise
-        return {
+
+        # Create result as a copy of the base context
+        result = copy.copy(context)
+        result.update({
             "variable": varname,
             "value": getattr(module, varname)
-        }
+        })
+        return result
 
     return payload
 
@@ -320,12 +327,14 @@ def create_run_function_payload(
                 real_kwargs[key] = value
                 initial_kwargs[key] = value
 
-        result = {
+        # Create result as a copy of the base context
+        result = copy.copy(context)
+        result.update({
             "value": fn(*real_posargs, **real_kwargs),
             "function": fname,
             "args": initial_posargs,
             "kwargs": initial_kwargs,
-        }
+        })
 
         if copy_args:
             result["used_args"] = real_posargs
@@ -434,13 +443,18 @@ def create_run_harness_payload(
 
             real_kwargs[key] = value
 
-        return {
+        # Create result as a copy of the base context
+        result = copy.copy(context)
+        result.update({
             "value": harness(fn, *real_posargs, **real_kwargs),
             "harness": harness,
             "function": fname,
             "args": real_posargs,
             "kwargs": real_kwargs
-        }
+        })
+
+        # Return our result
+        return result
 
     return payload
 
@@ -528,12 +542,17 @@ def create_execute_code_block_payload(block_name, src, nodes=None):
             exec(last_code, env)
             value = None
 
-        return {
+        # Create result as a copy of the base context
+        result = copy.copy(context)
+        result.update({
             "value": value,
             "block_name": block_name,
             "block": src,
             "block_nodes": nodes
-        }
+        })
+
+        # Return our result
+        return result
 
     return payload
 
@@ -771,7 +790,8 @@ def with_cleanup(payload, cleanup):
         Runs a base payload and then runs a cleanup function.
         """
         result = payload(context)
-        return cleanup(result)
+        result = cleanup(result)
+        return result
 
     return cleanup_payload
 
@@ -1787,6 +1807,10 @@ def report_argument_modifications(target, *args, **kwargs):
     arguments which are are lists or dictionaries at the top level, so
     its definition of modifiable is rather narrow.
 
+    The report uses argument positions when the test case is given
+    positional arguments and argument names when it's given keyword
+    arguments.
+
     (Note: the last two paragraphs of this docstring are picked up
     automatically as rubric values for tests using this harness. fname
     will be substituted in, which is why it appears in curly braces
@@ -1814,9 +1838,11 @@ def report_argument_modifications(target, *args, **kwargs):
         mkwdefaults = [k for k in target.__kwdefaults__ if k not in kwargs]
     else:
         mkwdefaults = []
-    nargs = target.__code__.co_argcount + target.__code__.co_kwonlyargcount
-    margnames = [target.__code__.co_varnames[:nargs][i] for i in mposargs]
-    mposnames = margnames[:len(mposargs)]
+    # This code could be used to get argument names for positional
+    # arguments, but we actually don't want them.
+    #nargs = target.__code__.co_argcount + target.__code__.co_kwonlyargcount
+    #margnames = [target.__code__.co_varnames[:nargs][i] for i in mposargs]
+    #mposnames = margnames[:len(mposargs)]
     mposvals = [copy.deepcopy(args[i]) for i in mposargs]
     mkwvals = [copy.deepcopy(kwargs[k]) for k in mkwargs]
     mkwdefvals = {
@@ -1833,10 +1859,9 @@ def report_argument_modifications(target, *args, **kwargs):
     # Changes in positional argument values
     for argindex, orig in zip(mposargs, mposvals):
         final = args[argindex]
-        name = mposnames[argindex]
-        result += "Your code {} the value of the '{}' argument.\n".format(
+        result += "Your code {} the value of the {} argument.\n".format(
             "modified" if orig != final else "did not modify",
-            name
+            phrasing.ordinal(argindex)
         )
 
     # Changes in keyword argument values
@@ -1879,7 +1904,9 @@ def returns_a_new_value(target, *args, **kwargs):
     (Note: the last two paragraphs of this docstring are picked up
     automatically as rubric values for tests using this harness. fname
     will be substituted in, which is why it appears in curly braces
-    below.)
+    below. This harness can also be used to ensure that a function
+    doesn't return a new value, in which case an alternate description
+    should be used.)
 
     Description:
 
