@@ -128,11 +128,27 @@ ALL_REASONABLE_COMPREHENSION_PATTERNS = [
 `potluck.mast` patterns for single, double, and triple list comprehensions
 with and without filtering.
 
-Note: does not include dictionary comprehensions (TODO: that?)
+Note: does not include dictionary comprehensions (see below)
 
 We will simply hope that students do not use quadruple comprehensions...
 """
 
+ALL_GENERATOR_EXPRESSION_PATTERNS = (
+    ALL_REASONABLE_COMPREHENSION_PATTERNS
+  + [
+      p.replace('[', '(').replace(']', ')')
+      for p in ALL_REASONABLE_COMPREHENSION_PATTERNS
+    ] # generator expressions
+  + [
+      p.replace('[ _', '{ _: _').replace(']', '}')
+      for p in ALL_REASONABLE_COMPREHENSION_PATTERNS
+    ] # dictionary comprehensions
+)
+"""
+Patterns for up to triple-loop list comprehensions, plus equivalent
+generator expressions and dictionary comprehensions.
+"""
+
 
 # Combination patterns for loops and loops + comprehensions
 ALL_FOR_AND_WHILE_LOOP_PATTERNS = ALL_FOR_PATTERNS + ALL_WHILE_PATTERNS
@@ -141,7 +157,7 @@ ALL_FOR_AND_WHILE_LOOP_PATTERNS = ALL_FOR_PATTERNS + ALL_WHILE_PATTERNS
 """
 ALL_LOOP_AND_COMPREHENSION_PATTERNS = (
     ALL_FOR_AND_WHILE_LOOP_PATTERNS
-  + ALL_REASONABLE_COMPREHENSION_PATTERNS
+  + ALL_GENERATOR_EXPRESSION_PATTERNS
 )
 """
 `potluck.mast` patterns for all reasonable loop/comprehension types,
@@ -208,6 +224,35 @@ the entire object + method expression, whereas these patterns will bind
 calls).
 """
 
+TRY_EXCEPT_PATTERNS = [
+    "try:\n  ___\nexcept:\n  ___",
+    "try:\n  ___\nexcept:\n  ___\nfinally:\n  ___",
+    "try:\n  ___\nexcept _:\n  ___",
+    "try:\n  ___\nexcept _:\n  ___\nfinally:\n  ___",
+    "try:\n  ___\nexcept _ as e:\n  ___",
+    "try:\n  ___\nexcept _ as e:\n  ___\nfinally:\n  ___",
+    "try:\n  ___\nfinally:\n  ___",
+]
+"""
+Patterns for try/except/finally blocks. Note that these do not include
+matching for a single try with multiple excepts nor do they allow
+matching variable exception names using 'as' (because that's not
+currently supported by `mast`).
+"""
+
+WITH_PATTERNS = [
+    "with _:\n  ___",
+    "with _, _:\n  ___",
+    "with _ as _:\n  ___",
+    "with _ as _, _ as _:\n  ___",
+    "with _ as _, _:\n  ___",
+    "with _, _ as _:\n  ___",
+]
+"""
+Patterns for with blocks with up to two context handlers (TODO: let there
+be an arbitrary number of them...).
+"""
+
 
 #-----------------------------#
 # Pattern-producing functions #
