@@ -257,6 +257,61 @@ def evaluation_key(course, semester, username, pset, phase, task):
 # Roster loading #
 #----------------#
 
+def get_variable_field_value(titles, row, name_options):
+    """
+    Extracts info from a CSV row that might be found in different columns
+    (or combinations of columns).
+
+    Needs a list of column titles (lowercased strings), plus a name
+    options list, and the row to extract from. Each item in the name
+    options list is either a string (column name in lowercase), or a
+    tuple containing one or more strings followed by a function to be
+    called with the values of those columns as arguments whose result
+    will be used as the value.
+
+    Returns the value from the row as extracted by the extraction
+    function, or simply looked up in the case of a string entry. Tries
+    options from the first to the last and returns the value from the
+    first one that worked.
+    """
+    obj = {}
+    val = obj
+    for opt in name_options:
+        if isinstance(opt, str):
+            try:
+                val = row[titles.index(opt)]
+            except ValueError:
+                continue
+        else:
+            try:
+                args = [row[titles.index(name)] for name in opt[:-1]]
+            except ValueError:
+                continue
+            val = opt[-1](*args)
+
+        if val is not obj:
+            return val
+
+    optitems = [
+        "'" + opt + "'"
+            if isinstance(opt, str)
+            else ' & '.join(["'" + o + "'" for o in opt[:-1]])
+        for opt in name_options
+    ]
+    if len(optitems) == 1:
+        optsummary = optitems[0]
+    elif len(optitems) == 2:
+        optsummary = optitems[0] + " or " + optitems[1]
+    else:
+        optsummary = ', '.join(optitems[:-1]) + ", or " + optitems[-1]
+    raise ValueError(
+        "Roster does not have column(s) {}. Columns are:\n  {}".format(
+            optsummary,
+            '\n  '.join(titles)
+        )
+    )
+
+
 def load_roster_from_stream(iterable_of_strings):
     """
     Implements the roster-loading logic given an iterable of strings,
@@ -270,27 +325,6 @@ def load_roster_from_stream(iterable_of_strings):
     # by default
     titles = next(reader) # Read first title line of roster
     titles = [x.lower() for x in titles] # convert columns to lower-case
-    emailIndex = titles.index('email')
-
-    if 'student name' in titles:
-        nameIndex = titles.index('student name')
-
-        def get_name(row):
-            return row[nameIndex]
-
-    else:
-        firstIndex = titles.index('first')
-        lastIndex = titles.index('last')
-
-        def get_name(row):
-            return "{} {}".format(row[firstIndex], row[lastIndex])
-
-    if 'section' in titles:
-        lecIndex = titles.index('section')
-    elif 'lec' in titles:
-        lecIndex = titles.index('lec')
-    else:
-        lecIndex = titles.index('course title')
 
     if "sort name" in titles:
         sortnameIndex = titles.index('sort name')
@@ -300,22 +334,49 @@ def load_roster_from_stream(iterable_of_strings):
         sortnameIndex = None
 
     for row in reader:
-        username = row[emailIndex].split('@')[0]
+        username = get_variable_field_value(
+            titles,
+            row,
+            [
+                ('email', lambda e: e.split('@')[0]),
+                'username'
+            ]
+        )
         if 0 < len(username):
-            name = get_name(row)
+            name = get_variable_field_value(
+                titles,
+                row,
+                [
+                    'name', 'student name',
+                    ('first', 'last', lambda f, l: f + ' ' + l),
+                    'sort name'
+                ]
+            )
+            section = get_variable_field_value(
+                titles,
+                row,
+                [
+                    'section',
+                    'lecture section',
+                    'lec',
+                    'lec sec',
+                    'lec section',
+                    'course_title',
+                ]
+            )
             namebits = name.split()
             if sortnameIndex is not None:
                 sort_by = row[sortnameIndex]
             else:
                 sort_by = ' '.join(
-                    [row[lecIndex], namebits[-1]]
+                    [section, namebits[-1]]
                     + namebits[:-1]
                 )
             students[username] = {
                 'username': username,
-                'fullname': get_name(row),
+                'fullname': name,
                 'sortname': sort_by,
-                'course_section': row[lecIndex]
+                'course_section': section
             }
         pass
     pass
